#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.interfaces import IBaseContentInfo
from pyams_content.shared.common.interfaces import IWfSharedContent, IWfSharedContentRoles, ISharedTool
from pyams_form.interfaces.form import IWidgetForm, IInnerTabForm, IInnerSubForm
from pyams_sequence.interfaces import ISequentialIdInfo, ISequentialIntIds
from pyams_skin.interfaces import IInnerPage
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION
from pyams_workflow.interfaces import IWorkflowState, IWorkflow, IWorkflowPublicationInfo

# import packages
from pyams_content.shared.common.zmi.header import SharedContentHeaderContentProvider
from pyams_pagelet.pagelet import pagelet_config
from pyams_utils.adapter import adapter_config
from pyams_utils.date import format_datetime
from pyams_utils.registry import get_utility
from pyams_utils.timezone import tztime
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import contentprovider_config
from pyams_zmi.form import AdminDisplayForm, InnerAdminDisplayForm
from z3c.form import field
from zope.interface import implementer, Interface

from pyams_content import _


@pagelet_config(name='summary.html', context=IWfSharedContent, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
@implementer(IWidgetForm, IInnerPage)
class SharedContentSummaryForm(AdminDisplayForm):
    """Shared content summary form"""

    legend = _("Display content summary")
    css_class = 'ams-form form-horizontal form-tight'

    fields = field.Fields(Interface)


@contentprovider_config(name='content_header', context=IWfSharedContent, view=SharedContentSummaryForm,
                        layer=IPyAMSLayer)
class SharedContentSummaryFormHeaderProvider(SharedContentHeaderContentProvider):
    """Shared content summary form header provider"""

    @property
    def back_url(self):
        tool = get_parent(self.context, ISharedTool)
        return absolute_url(tool, self.request, 'admin')


@adapter_config(name='dublincore-summary',
                context=(IWfSharedContent, IPyAMSLayer, SharedContentSummaryForm),
                provides=IInnerTabForm)
class SharedContentDublinCoreSummary(InnerAdminDisplayForm):
    """Shared content DublinCore summary"""

    weight = 1
    tab_label = _("Identity card")
    css_class = 'form-tight'

    fields = field.Fields(Interface)


@adapter_config(name='dublincore-version-summary',
                context=(IWfSharedContent, IPyAMSLayer, SharedContentDublinCoreSummary),
                provides=IInnerSubForm)
class SharedContentDublinCoreVersionSummary(InnerAdminDisplayForm):
    """Shared content version DublinCore summary"""

    legend = _("About this version")
    weight = 1

    @property
    def fields(self):
        fields = field.Fields(ISequentialIdInfo).select('hex_oid') + \
                 field.Fields(IWorkflowState).select('version_id') + \
                 field.Fields(IWfSharedContentRoles).select('owner') + \
                 field.Fields(IWfSharedContent).select('creator') + \
                 field.Fields(IBaseContentInfo).select('created_date') + \
                 field.Fields(IWorkflowPublicationInfo).select('publication') + \
                 field.Fields(IBaseContentInfo).select('modified_date') + \
                 field.Fields(IWfSharedContent).select('last_modifier', 'modifiers')
        workflow = IWorkflow(self.context)
        if IWorkflowState(self.context).state not in workflow.published_states:
            fields = fields.omit('publication')
        return fields

    def updateWidgets(self, prefix=None):
        super(SharedContentDublinCoreVersionSummary, self).updateWidgets(prefix)
        sequence = get_utility(ISequentialIntIds)
        self.widgets['hex_oid'].value = sequence.get_short_oid(ISequentialIdInfo(self.context).oid)
        workflow = IWorkflow(self.context)
        self.widgets['version_id'].label = _("Version")
        self.widgets['version_id'].value = '{version_id} - {status}'.format(
            version_id=IWorkflowState(self.context).version_id,
            status=self.request.localizer.translate(workflow.get_state_label(IWorkflowState(self.context).state)))
        info = IBaseContentInfo(self.context)
        self.widgets['created_date'].value = format_datetime(tztime(info.created_date))
        self.widgets['modified_date'].value = format_datetime(tztime(info.modified_date))


@adapter_config(name='dublincore-history-summary',
                context=(IWfSharedContent, IPyAMSLayer, SharedContentDublinCoreSummary),
                provides=IInnerSubForm)
class SharedContentDublinCoreHistorySummary(InnerAdminDisplayForm):
    """Shared content history DublinCore summary"""

    legend = _("Content history")

    fields = field.Fields(IWfSharedContent).select('first_owner') + \
             field.Fields(IWorkflowPublicationInfo).select('first_publication_date')

    weight = 2

    def updateWidgets(self, prefix=None):
        super(SharedContentDublinCoreHistorySummary, self).updateWidgets(prefix)
        info = IWorkflowPublicationInfo(self.context)
        self.widgets['first_publication_date'].value = format_datetime(tztime(info.first_publication_date))
